@@ -50,6 +50,10 @@
 package com.openexchange.filestore.s3.internal;
 
 import static com.openexchange.filestore.s3.internal.S3ExceptionCode.wrap;
+import static com.openexchange.java.Autoboxing.L;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.IOException;
 import java.io.InputStream;
 import java.io.SequenceInputStream;
 import java.util.ArrayList;
@@ -86,6 +90,7 @@ import com.amazonaws.services.s3.model.UploadPartResult;
 import com.openexchange.exception.OXException;
 import com.openexchange.filestore.FileStorage;
 import com.openexchange.filestore.FileStorageCodes;
+import com.openexchange.filestore.utils.TempFileHelper;
 import com.openexchange.java.Streams;
 import com.openexchange.java.Strings;
 import com.openexchange.java.util.UUIDs;
@@ -137,15 +142,28 @@ public class S3FileStorage implements FileStorage {
     }
 
     @Override
-    public String saveNewFile(InputStream file) throws OXException {
+    public String saveNewFile(InputStream input) throws OXException {
         /*
          * perform chunked upload as needed
          */
         String key = generateKey(true);
         S3ChunkedUpload chunkedUpload = null;
         S3UploadChunk chunk = null;
+        File tmpFile = null;
         try {
-            chunkedUpload = new S3ChunkedUpload(file, clientSideEncryption, chunkSize);
+            /*
+             * spool to file
+             */
+            if (!(input instanceof FileInputStream)) {
+                tmpFile = TempFileHelper.getInstance().newTempFile();
+                if (tmpFile != null) {
+                    input = Streams.transferToFileAndCreateStream(input, tmpFile);
+                }
+            }
+            /*
+             * proceed
+             */
+            chunkedUpload = new S3ChunkedUpload(input, clientSideEncryption, chunkSize);
             chunk = chunkedUpload.next();
             if (false == chunkedUpload.hasNext()) {
                 /*
@@ -185,8 +203,11 @@ public class S3FileStorage implements FileStorage {
                     }
                 }
             }
+        } catch (IOException e) {
+            throw FileStorageCodes.IOERROR.create(e, e.getMessage());
         } finally {
-            Streams.close(chunk, chunkedUpload, file);
+            Streams.close(chunk, chunkedUpload, input);
+            TempFileHelper.deleteQuietly(tmpFile);
         }
         return removePrefix(key);
     }
@@ -200,7 +221,7 @@ public class S3FileStorage implements FileStorage {
     public InputStream getFile(String name, long offset, long length) throws OXException {
         long fileSize = getFileSize(name);
         if (offset >= fileSize || length >= 0 && length > fileSize - offset) {
-            throw FileStorageCodes.INVALID_RANGE.create(offset, length, name, fileSize);
+            throw FileStorageCodes.INVALID_RANGE.create(L(offset), L(length), name, L(fileSize));
         }
         String key = addPrefix(name);
         GetObjectRequest request = new GetObjectRequest(bucketName, key);
@@ -213,7 +234,7 @@ public class S3FileStorage implements FileStorage {
             return new AbortIfNotFullyConsumedS3ObjectInputStreamWrapper(amazonS3.getObject(request).getObjectContent());
         } catch (AmazonClientException e) {
             if (AmazonServiceException.class.isInstance(e) && HttpServletResponse.SC_REQUESTED_RANGE_NOT_SATISFIABLE == ((AmazonServiceException) e).getStatusCode()) {
-                throw FileStorageCodes.INVALID_RANGE.create(e, offset, length, name, fileSize);
+                throw FileStorageCodes.INVALID_RANGE.create(e, L(offset), L(length), name, L(fileSize));
             }
             throw wrap(e, key);
         }
@@ -322,6 +343,7 @@ public class S3FileStorage implements FileStorage {
 
     @Override
     public long appendToFile(InputStream file, String name, long offset) throws OXException {
+        File tmpFile = null;
         try {
             /*
              * TODO: This would be more efficient using the "CopyPartRequest", which is not yet supported by ceph
@@ -340,6 +362,15 @@ public class S3FileStorage implements FileStorage {
                 if (currentLength != offset) {
                     throw FileStorageCodes.INVALID_OFFSET.create(Long.valueOf(offset), name, Long.valueOf(currentLength));
                 }
+                /*
+                 * spool to file
+                 */
+                if (!(file instanceof FileInputStream)) {
+                    tmpFile = TempFileHelper.getInstance().newTempFile();
+                    if (tmpFile != null) {
+                        file = Streams.transferToFileAndCreateStream(file, tmpFile);
+                    }
+                }
                 /*
                  * append both streams at temporary location
                  */
@@ -363,8 +394,11 @@ public class S3FileStorage implements FileStorage {
             } catch (AmazonClientException e) {
                 throw wrap(e, key);
             }
+        } catch (IOException e) {
+            throw FileStorageCodes.IOERROR.create(e, e.getMessage());
         } finally {
             Streams.close(file);
+            TempFileHelper.deleteQuietly(tmpFile);
         }
     }
 
