@@ -63,8 +63,8 @@ import java.security.PublicKey;
 import java.security.UnrecoverableKeyException;
 import java.security.cert.Certificate;
 import java.security.cert.CertificateException;
+import java.util.Collections;
 import java.util.Enumeration;
-import java.util.HashMap;
 import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
@@ -152,24 +152,24 @@ public class S3FileStorageFactory implements FileStorageProvider {
     @Override
     public S3FileStorage getFileStorage(URI uri) throws OXException {
         try {
-                LOG.debug("Initializing S3 client for {}", uri);
-                /*
-                 * extract filestore ID from authority part of URI
-                 */
-                String filestoreID = extractFilestoreID(uri);
-                LOG.debug("Using \"{}\" as filestore ID.", filestoreID);
-                /*
-                 * create client
-                 */
-                LeanConfigurationService configService = services.getServiceSafe(LeanConfigurationService.class);
-                Map<String, String> optional = getOptional(filestoreID);
-                String encryption = getPropertySafe(S3Properties.ENCRYPTION, configService, optional);
-                S3EncryptionConfig s3EncryptionConfig = new S3EncryptionConfig(encryption);
-                AmazonS3ClientInfo clientInfo = initClient(filestoreID, configService, s3EncryptionConfig);
-                AmazonS3Client client = clientInfo.client;
-                String bucketName = initBucket(client, filestoreID, configService, s3EncryptionConfig);
-                LOG.debug("Using \"{}\" as bucket name.", bucketName);
-                return new S3FileStorage(client, clientInfo.encrypted, s3EncryptionConfig.getServerSideEncryption() != null, bucketName, extractFilestorePrefix(uri), clientInfo.chunkSize);
+            LOG.debug("Initializing S3 client for {}", uri);
+            /*
+             * extract filestore ID from authority part of URI
+             */
+            String filestoreID = extractFilestoreID(uri);
+            LOG.debug("Using \"{}\" as filestore ID.", filestoreID);
+            /*
+             * create client
+             */
+            LeanConfigurationService configService = services.getServiceSafe(LeanConfigurationService.class);
+            Map<String, String> optional = getOptional(filestoreID);
+            String encryption = getPropertySafe(S3Properties.ENCRYPTION, configService, optional);
+            S3EncryptionConfig s3EncryptionConfig = new S3EncryptionConfig(encryption);
+            AmazonS3ClientInfo clientInfo = initClient(filestoreID, configService, s3EncryptionConfig);
+            AmazonS3Client client = clientInfo.client;
+            String bucketName = initBucket(client, filestoreID, configService, s3EncryptionConfig);
+            LOG.debug("Using \"{}\" as bucket name.", bucketName);
+            return new S3FileStorage(client, clientInfo.encrypted, s3EncryptionConfig.getServerSideEncryption() != null, bucketName, extractFilestorePrefix(uri), clientInfo.chunkSize);
         } catch (OXException ex) {
             Throwable cause = ex.getCause();
             if (cause instanceof AmazonS3Exception) {
@@ -205,7 +205,7 @@ public class S3FileStorageFactory implements FileStorageProvider {
      * @param filestoreID The filestore ID
      * @param configService The {@link LeanConfigurationService}
      * @param encryptionConfig The {@link S3EncryptionConfig} of the given filestore
-     * @return The client
+     * @return The {@link AmazonS3ClientInfo} containing the configured {@link AmazonS3Client}
      * @throws OXException
      */
     private AmazonS3ClientInfo initClient(String filestoreID, LeanConfigurationService configService, S3EncryptionConfig encryptionConfig) throws OXException {
@@ -269,7 +269,16 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return new AmazonS3ClientInfo((AmazonS3Client) clientBuilder.build(), encrypted, chunkSize);
     }
 
-    private String getPropertySafe(S3Properties prop, LeanConfigurationService configurationService, Map<String, String> optionals) throws OXException {
+    /**
+     * Gets the property safe
+     *
+     * @param prop The property to retrieve
+     * @param configurationService The {@link LeanConfigurationService} to use
+     * @param optionals The optionals to use
+     * @return The property
+     * @throws OXException
+     */
+    private static String getPropertySafe(S3Properties prop, LeanConfigurationService configurationService, Map<String, String> optionals) throws OXException {
         String property = configurationService.getProperty(prop, optionals);
         if (Strings.isEmpty(property)) {
             throw ConfigurationExceptionCodes.PROPERTY_MISSING.create(prop.getFQPropertyName(optionals));
@@ -277,18 +286,68 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return property;
     }
 
-    private Map<String, String> getOptional(String filestoreId) {
-        HashMap<String, String> hashMap = new HashMap<>();
-        hashMap.put(S3Properties.OPTIONAL_NAME, filestoreId);
-        return hashMap;
+    /**
+     * Creates and optional map containing only the given filestore id
+     *
+     * @param filestoreId The filestore id
+     * @return The optional map
+     */
+    private static Map<String, String> getOptional(String filestoreId) {
+        return Collections.singletonMap(S3Properties.OPTIONAL_NAME, filestoreId);
     }
 
+    /**
+     * Gets the client configuration for the given filestore id
+     *
+     * @param filestoreID The filestore id
+     * @param configService The {@link LeanConfigurationService} to use
+     * @return The {@link ClientConfiguration} for the filestore id
+     */
     private ClientConfiguration getClientConfiguration(String filestoreID, LeanConfigurationService configService) {
         ClientConfiguration clientConfiguration = new ClientConfiguration();
-        String signerOverride = configService.getProperty(S3Properties.SIGNER_OVERRIDE, getOptional(filestoreID));
 
-        if (Strings.isNotEmpty(signerOverride)) {
-            clientConfiguration.setSignerOverride(signerOverride);
+        Map<String, String> optional = getOptional(filestoreID);
+        {
+            String signerOverride = configService.getProperty(S3Properties.SIGNER_OVERRIDE, optional);
+            if (Strings.isNotEmpty(signerOverride)) {
+                clientConfiguration.setSignerOverride(signerOverride);
+            }
+        }
+
+        {
+            String connectTimeout = configService.getProperty(S3Properties.CONNECT_TIMEOUT, optional);
+            if (Strings.isNotEmpty(connectTimeout)) {
+                try {
+                    clientConfiguration.setConnectionTimeout(Integer.parseInt(connectTimeout.trim()));
+                } catch (NumberFormatException e) {
+                    // Invalid connect timeout.
+                    LOG.warn("Invalid integer value specified for {}", S3Properties.CONNECT_TIMEOUT.getFQPropertyName(), e);
+                }
+            }
+        }
+
+        {
+            String readTimeout = configService.getProperty(S3Properties.READ_TIMEOUT, optional);
+            if (Strings.isNotEmpty(readTimeout)) {
+                try {
+                    clientConfiguration.setSocketTimeout(Integer.parseInt(readTimeout.trim()));
+                } catch (NumberFormatException e) {
+                    // Invalid connect timeout.
+                    LOG.warn("Invalid integer value specified for {}", S3Properties.READ_TIMEOUT.getFQPropertyName(), e);
+                }
+            }
+        }
+
+        {
+            String maxConnectionPoolSize = configService.getProperty(S3Properties.MAX_CONNECTION_POOL_SIZE, optional);
+            if (Strings.isNotEmpty(maxConnectionPoolSize)) {
+                try {
+                    clientConfiguration.setMaxConnections(Integer.parseInt(maxConnectionPoolSize.trim()));
+                } catch (NumberFormatException e) {
+                    // Invalid connect timeout.
+                    LOG.warn("Invalid integer value specified for {}", S3Properties.MAX_CONNECTION_POOL_SIZE.getFQPropertyName(), e);
+                }
+            }
         }
 
         String proxyHost = System.getProperty("http.proxyHost");
@@ -315,9 +374,18 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return clientConfiguration;
     }
 
-    private EncryptionMaterials getEncryptionMaterials(String filestoreID, EncryptionType clientType, LeanConfigurationService configurationService) throws OXException {
-        if (!EncryptionType.RSA.equals(clientType)) {
-            throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create("Unsupported encryption type: " + clientType.getName());
+    /**
+     * Creates a new {@link EncryptionMaterials} for the given filestore id and also checks if the client encryption type is applicable.
+     *
+     * @param filestoreID The filestore id
+     * @param clientEncryptionType The {@link EncryptionType} of the client
+     * @param configurationService The {@link LeanConfigurationService} to use
+     * @return The {@link EncryptionMaterials}
+     * @throws OXException in case of errors or if the client encryption type is not applicable
+     */
+    private EncryptionMaterials getEncryptionMaterials(String filestoreID, EncryptionType clientEncryptionType, LeanConfigurationService configurationService) throws OXException {
+        if (!EncryptionType.RSA.equals(clientEncryptionType)) {
+            throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create("Unsupported encryption type: " + clientEncryptionType.getName());
         }
         Map<String, String> optional = getOptional(filestoreID);
         String keyStore = getPropertySafe(S3Properties.RSA_KEYSTORE, configurationService, optional);
@@ -405,7 +473,7 @@ public class S3FileStorageFactory implements FileStorageProvider {
         }
 
         if (false == bucketExists) {
-            String region = configService.getProperty(S3Properties.REGION, getOptional(filestoreID));
+            String region = configService.getProperty(S3Properties.REGION, optional);
 
             try {
                 s3client.createBucket(new CreateBucketRequest(bucketName, Region.fromValue(region)));
@@ -530,6 +598,12 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return null == sb ? path : sb.toString();
     }
 
+    /**
+     * Checks if the character is an allowed special character
+     *
+     * @param ch The character to check
+     * @return true if it is an allowed special character, false otherwise
+     */
     private static boolean isAllowedSpecial(char ch) {
         switch (ch) {
             case '!':
@@ -567,6 +641,12 @@ public class S3FileStorageFactory implements FileStorageProvider {
         return authority;
     }
 
+    /**
+     *
+     * {@link AmazonS3ClientInfo}
+     *
+     * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
+     */
     private static class AmazonS3ClientInfo {
 
         /** The Amazon S3 client reference */
